//=============================================================================
//
// file :        AH401B.h
//
// description : Include for the AH401B class.
//
// project :	AH401B TANGO DS
//
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza (TS)
//              ITALY
//
//-=============================================================================
//
//              This file is generated by POGO
//      (Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF 
//=============================================================================
#ifndef _AH401B_H
#define _AH401B_H

#include <tango.h>
//using namespace Tango;

#include <sys/time.h>
#include <time.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <buffer.h>
#include "picoammeter.h"

#define AH401B_TIMEOUT 1000000 /* data old timeout (usec) */
#define AH401B_TOUT_SEC  2    /* timeout on udp socket (sec) */
#define AH401B_NUM_RANGE 8
//#define AH401B_NUM_CHAN  4

//#define AH401B_UDP_LOCAL_PORT  10011

// 0xA
#define LF 10
// 0xD
#define CR 13

// microseconds
#define STATE_TOUT   1800	//30 min
#define STATE_TOUT_2   60	//1/2 min
#define HV_TOUT		   60		//30 sec

//server status array
#define STATUSARRAY_LENGTH 8
#define ACQ 0
#define BDR 1
#define BIN 2
#define HLF 3
#define ITM 4
#define RNG 5
#define TRG 6
#define NAQ 7

#define COMM_ERROR		0x1
#define COMM_OK			0x2

#define STRING_SIZE 256
const short BUF_LEN=5000;

static const char *ranges[]=
{
	"1.8 nC",
	" 50 pC",
	"100 pC",
	"150 pC",
	"200 pC",
	"250 pC",
	"300 pC",
	"350 pC"
};

static int val_ranges[]=		//pC
{
	1800,
	50,
	100,
	150,
	200,
	250,
	300,
	350
};

// class to acquire indefinitley
// (TRG ON mode))
//class AcquireLoop;

/**
 * @author	$Author: graziano $
 * @version	$Revision: 1.12 $
 */

 //	Add your own constants definitions here.
 //-----------------------------------------------


namespace AH401B_ns
{

/**
 * Class Description:
 * TANGO server interfacing ELETTRA AH401B picoammeter
 */

/*
 *	Device States Description:
*  Tango::RUNNING :  Device is in continuous acquisition
*  Tango::ON :       Device is responding on socket
*  Tango::FAULT :    Device doesn't respond on socket, or other errors occurred
 */


class AH401B: public TANGO_BASE_CLASS
{
public :
	//	Add your own data members here
	//-----------------------------------------


	//	Here is the Start of the automatic code generation part
	//-------------------------------------------------------------	
/**
 *	@name attributes
 *	Attributs member data.
 */
//@{
		Tango::DevDouble	*attr_CH1Data_read;
		Tango::DevDouble	*attr_CH2Data_read;
		Tango::DevDouble	*attr_CH3Data_read;
		Tango::DevDouble	*attr_CH4Data_read;
		Tango::DevDouble	*attr_CH1_read;
		Tango::DevDouble	*attr_CH2_read;
		Tango::DevDouble	*attr_CH3_read;
		Tango::DevDouble	*attr_CH4_read;
		Tango::DevDouble	*attr_CH1Offset_read;
		Tango::DevDouble	attr_CH1Offset_write;
		Tango::DevDouble	*attr_CH2Offset_read;
		Tango::DevDouble	attr_CH2Offset_write;
		Tango::DevDouble	*attr_CH3Offset_read;
		Tango::DevDouble	attr_CH3Offset_write;
		Tango::DevDouble	*attr_CH4Offset_read;
		Tango::DevDouble	attr_CH4Offset_write;
		Tango::DevLong	*attr_Alarm1_read;
		Tango::DevLong	attr_Alarm1_write;
		Tango::DevLong	*attr_Alarm2_read;
		Tango::DevLong	attr_Alarm2_write;
		Tango::DevDouble	*attr_IntegrationTime_read;
		Tango::DevDouble	attr_IntegrationTime_write;
		Tango::DevLong	*attr_Range_read;
		Tango::DevLong	attr_Range_write;
		Tango::DevLong	*attr_NumAcq_read;
		Tango::DevLong	attr_NumAcq_write;
		Tango::DevBoolean	*attr_Background_read;
		Tango::DevBoolean	attr_Background_write;
		Tango::DevBoolean	*attr_Calibration_read;
		Tango::DevBoolean	attr_Calibration_write;
		Tango::DevLong	*attr_BunchNumber_read;
		Tango::DevLong	*attr_PAStatus_read;
		Tango::DevDouble	*attr_HighVoltage_read;
		Tango::DevDouble	attr_HighVoltage_write;
		Tango::DevDouble	*attr_HighVoltageSet_read;
//@}

/**
 *	@name Device properties
 *	Device properties member data.
 */
//@{
/**
 *	
 */
	string	bufferCH1Device;
/**
 *	
 */
	string	bufferCH2Device;
/**
 *	
 */
	string	bufferCH3Device;
/**
 *	
 */
	string	bufferCH4Device;
/**
 *	Calibration array (proportional value) for each range value [0..7]
 *	Ex:
 *	1.0:2.0:3.0:4.0:5.0:6.0:7.0:8.0
 */
	string	cH1Cal;
/**
 *	
 */
	string	cH2Cal;
/**
 *	
 */
	string	cH3Cal;
/**
 *	
 */
	string	cH4Cal;
/**
 *	
 */
	Tango::DevLong	integrationTime;
/**
 *	Number of samples to keep in memory, in order to retrieve with getSamples command.
 */
	Tango::DevLong	nSamples;
/**
 *	Number of acquisitions per trigger
 */
	Tango::DevLong	numAcq;
/**
 *	
 */
	Tango::DevLong	range;
/**
 *	Backgroud level
 *	ch1:ch2:ch3:ch4
 */
	string	backgroundLevel;
/**
 *	Picoammeter index for the RT module
 */
	Tango::DevUShort	picoammeterIndex;
/**
 *	Index in rnm for the RT module
 */
	Tango::DevShort	rnmIndex;
/**
 *	IP address of Picoammeter
 */
	string	picoammeterIP;
/**
 *	MAC address of Picoammeter
 */
	string	picoammeterMAC;
/**
 *	UDP port of Picoammeter
 */
	Tango::DevShort	picoammeterPort;
/**
 *	IP address of CPU
 */
	string	cpuIP;
/**
 *	UDP port of CPU
 */
	Tango::DevShort	cpuPort;
/**
 *	If not triggered TRG OFF, ACQ ON and NAQ 0, else TRG ON, ACQ OFF, NAQ 1
 */
	Tango::DevLong	triggered;
/**
 *	If HalfMode HLF ON (only one integrator circuit running, half speed, bet noise) else HLF OFF (two integrator circuits)
 */
	Tango::DevLong	halfMode;
/**
 *	Use Xpi commands
 */
	Tango::DevLong	xpiMode;
//@}

/**@name Constructors
 * Miscellaneous constructors */
//@{
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	AH401B(Tango::DeviceClass *cl,string &s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	AH401B(Tango::DeviceClass *cl,const char *s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device name
 *	@param d	Device description.
 */
	AH401B(Tango::DeviceClass *cl,const char *s,const char *d);
//@}

/**@name Destructor
 * Only one desctructor is defined for this class */
//@{
/**
 * The object desctructor.
 */	
	~AH401B() {delete_device();};
/**
 *	will be called at device destruction or at init command.
 */
	void delete_device();
//@}

	
/**@name Miscellaneous methods */
//@{
/**
 *	Initialize the device
 */
	virtual void init_device();
/**
 *	Always executed method befor execution command method.
 */
	virtual void always_executed_hook();

//@}

/**
 * @name AH401B methods prototypes
 */

//@{
/**
 *	Hardware acquisition for attributes.
 */
	virtual void read_attr_hardware(vector<long> &attr_list);
/**
 *	Extract real attribute values for CH1Data acquisition result.
 */
	virtual void read_CH1Data(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CH2Data acquisition result.
 */
	virtual void read_CH2Data(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CH3Data acquisition result.
 */
	virtual void read_CH3Data(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CH4Data acquisition result.
 */
	virtual void read_CH4Data(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CH1 acquisition result.
 */
	virtual void read_CH1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CH2 acquisition result.
 */
	virtual void read_CH2(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CH3 acquisition result.
 */
	virtual void read_CH3(Tango::Attribute &attr);
/**
 *	Extract real attribute values for CH1Offset acquisition result.
 */
	virtual void read_CH1Offset(Tango::Attribute &attr);
/**
 *	Write CH1Offset attribute values to hardware.
 */
	virtual void write_CH1Offset(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for CH2Offset acquisition result.
 */
	virtual void read_CH2Offset(Tango::Attribute &attr);
/**
 *	Write CH2Offset attribute values to hardware.
 */
	virtual void write_CH2Offset(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for CH3Offset acquisition result.
 */
	virtual void read_CH3Offset(Tango::Attribute &attr);
/**
 *	Write CH3Offset attribute values to hardware.
 */
	virtual void write_CH3Offset(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for CH4Offset acquisition result.
 */
	virtual void read_CH4Offset(Tango::Attribute &attr);
/**
 *	Write CH4Offset attribute values to hardware.
 */
	virtual void write_CH4Offset(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for CH4 acquisition result.
 */
	virtual void read_CH4(Tango::Attribute &attr);
/**
 *	Extract real attribute values for IntegrationTime acquisition result.
 */
	virtual void read_IntegrationTime(Tango::Attribute &attr);
/**
 *	Write IntegrationTime attribute values to hardware.
 */
	virtual void write_IntegrationTime(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Range acquisition result.
 */
	virtual void read_Range(Tango::Attribute &attr);
/**
 *	Write Range attribute values to hardware.
 */
	virtual void write_Range(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for NumAcq acquisition result.
 */
	virtual void read_NumAcq(Tango::Attribute &attr);
/**
 *	Write NumAcq attribute values to hardware.
 */
	virtual void write_NumAcq(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Background acquisition result.
 */
	virtual void read_Background(Tango::Attribute &attr);
/**
 *	Write Background attribute values to hardware.
 */
	virtual void write_Background(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Calibration acquisition result.
 */
	virtual void read_Calibration(Tango::Attribute &attr);
/**
 *	Write Calibration attribute values to hardware.
 */
	virtual void write_Calibration(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for BunchNumber acquisition result.
 */
	virtual void read_BunchNumber(Tango::Attribute &attr);
/**
 *	Extract real attribute values for PAStatus acquisition result.
 */
	virtual void read_PAStatus(Tango::Attribute &attr);
/**
 *	Extract real attribute values for HighVoltage acquisition result.
 */
	virtual void read_HighVoltage(Tango::Attribute &attr);
/**
 *	Write HighVoltage attribute values to hardware.
 */
	virtual void write_HighVoltage(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for HighVoltageSet acquisition result.
 */
	virtual void read_HighVoltageSet(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Alarm1 acquisition result.
 */
	virtual void read_Alarm1(Tango::Attribute &attr);
/**
 *	Write Alarm1 attribute values to hardware.
 */
	virtual void write_Alarm1(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Alarm2 acquisition result.
 */
	virtual void read_Alarm2(Tango::Attribute &attr);
/**
 *	Write Alarm2 attribute values to hardware.
 */
	virtual void write_Alarm2(Tango::WAttribute &attr);
/**
 *	Read/Write allowed for CH1Data attribute.
 */
	virtual bool is_CH1Data_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH2Data attribute.
 */
	virtual bool is_CH2Data_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH3Data attribute.
 */
	virtual bool is_CH3Data_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH4Data attribute.
 */
	virtual bool is_CH4Data_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH1 attribute.
 */
	virtual bool is_CH1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH2 attribute.
 */
	virtual bool is_CH2_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH3 attribute.
 */
	virtual bool is_CH3_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH4 attribute.
 */
	virtual bool is_CH4_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH1Offset attribute.
 */
	virtual bool is_CH1Offset_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH2Offset attribute.
 */
	virtual bool is_CH2Offset_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH3Offset attribute.
 */
	virtual bool is_CH3Offset_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for CH4Offset attribute.
 */
	virtual bool is_CH4Offset_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for IntegrationTime attribute.
 */
	virtual bool is_IntegrationTime_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Range attribute.
 */
	virtual bool is_Range_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for NumAcq attribute.
 */
	virtual bool is_NumAcq_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Background attribute.
 */
	virtual bool is_Background_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Calibration attribute.
 */
	virtual bool is_Calibration_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for BunchNumber attribute.
 */
	virtual bool is_BunchNumber_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for PAStatus attribute.
 */
	virtual bool is_PAStatus_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for HighVoltage attribute.
 */
	virtual bool is_HighVoltage_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for HighVoltageSet attribute.
 */
	virtual bool is_HighVoltageSet_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Alarm1 attribute.
 */
	virtual bool is_Alarm1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Alarm2 attribute.
 */
	virtual bool is_Alarm2_allowed(Tango::AttReqType type);
/**
 *	Execution allowed for SendCommand command.
 */
	virtual bool is_SendCommand_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetSnapshot command.
 */
	virtual bool is_GetSnapshot_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetSamples command.
 */
	virtual bool is_GetSamples_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Start command.
 */
	virtual bool is_Start_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Stop command.
 */
	virtual bool is_Stop_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCH1Data command.
 */
	virtual bool is_GetCH1Data_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCH2Data command.
 */
	virtual bool is_GetCH2Data_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCH3Data command.
 */
	virtual bool is_GetCH3Data_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCH4Data command.
 */
	virtual bool is_GetCH4Data_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCH1 command.
 */
	virtual bool is_GetCH1_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCH2 command.
 */
	virtual bool is_GetCH2_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCH3 command.
 */
	virtual bool is_GetCH3_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetCH4 command.
 */
	virtual bool is_GetCH4_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AcquireBackground command.
 */
	virtual bool is_AcquireBackground_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Reset command.
 */
	virtual bool is_Reset_allowed(const CORBA::Any &any);
/**
 * Send any command you want and see what happens.
 *	@param	argin	Command string
 *	@return	Reply
 *	@exception DevFailed
 */
	Tango::DevString	send_command(Tango::DevString);
/**
 * Read a single snapshot (4 channels)
 *	@return	The array with the 4 current measurements
 *	@exception DevFailed
 */
	Tango::DevVarLongArray	*get_snapshot();
/**
 * Returns last acquired samples (length depends on NSamples property).
 *	@param	argin	Fake. Use 1 and get all 4 channels
 *	@return	Array of samples
 *	@exception DevFailed
 */
	Tango::DevVarULongArray	*get_samples(Tango::DevUShort);
/**
 * Start acquisition on trigger.
 *	@return	
 *	@exception DevFailed
 */
	void	start();
/**
 * Stop acquisition on trigger.
 *	@return	
 *	@exception DevFailed
 */
	void	stop();
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_ch1_data(const Tango::DevVarLongArray *);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_ch2_data(const Tango::DevVarLongArray *);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_ch3_data(const Tango::DevVarLongArray *);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_ch4_data(const Tango::DevVarLongArray *);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_ch1(const Tango::DevVarLongArray *);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_ch2(const Tango::DevVarLongArray *);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_ch3(const Tango::DevVarLongArray *);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarDoubleArray	*get_ch4(const Tango::DevVarLongArray *);
/**
 * Acquire background level
 *	@exception DevFailed
 */
	void	acquire_background(Tango::DevLong);
/**
 * Reset
 *	@exception DevFailed
 */
	void	reset();

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

	//	Here is the end of the automatic code generation part
	//-------------------------------------------------------------	

friend class AcquireLoop;

private:
	string host_rw;
	long port_rw;

// private:
//workaround per un (probabile?) bug del compilatore rispetto alle friend class
	public :
		void                    openSocket();
		void                    sendCmd(Tango::DevString);
		int                     getResponse();
		void                    update_StatArray(bool flag_itm_upd, bool flag_rng_upd, bool flag_naq_upd, bool flag_alarm1_upd, bool flag_alarm2_upd);
		void                    update_HV(bool flag_hvs_upd, bool flag_hvs_set_upd);
		struct sockaddr_in      serv_addr;
		int                     sockfd;
		struct timeval          tv;
		struct timeval          last_state_time;
		struct timeval          last_hv_time;
		struct timeval          last_hv_set_time;
		void                    fileLogMsg(char *);
		char                    recvBuf[BUF_LEN];
		char                    response[BUF_LEN];
		char                    tmp[STRING_SIZE];
		
		Tango::DevBoolean       abort_acquire;
		Tango::DevLong          statusArray[STATUSARRAY_LENGTH];
		
		Tango::DevVarULongArray *samplesBuffer1;
		Tango::DevVarULongArray *samplesBuffer2;
		Tango::DevVarULongArray *samplesBuffer3;
		Tango::DevVarULongArray *samplesBuffer4;
		int                     samples_index;

		double cal_matrix[PICO_NUM_CHAN][AH401B_NUM_RANGE];		
		double background_level[PICO_NUM_CHAN];
		bool get_configuration(void);
		void string_explode(string, string, vector<string>*);
		void string_explode(string, string, vector<unsigned int>*, bool b_hex = false);
	
		int fd_socket;
		bool init_ah401b(void); 
//
	 protected :
		Tango::DevString write_read(char *);
		
		bool init_buffers(void);
		void delete_buffers(void);
		int fd_buffer[PICO_NUM_CHAN];
		struct _buffer *buf[PICO_NUM_CHAN];
		long vsize;
		//void check_get_data_param(const Tango::DevVarLongArray *, long *, long *, unsigned char *);

		picoammeter_shm_t *picoammeter;
		unsigned char picoammeter_ip[4];
		unsigned char picoammeter_mac[6];


		void	set_integration_time(Tango::DevDouble);
		void	set_full_scale_range(Tango::DevUShort);
		void	set_num_acq(Tango::DevLong);
		void	set_alarm1(Tango::DevLong);
		void	set_alarm2(Tango::DevLong);
		void	set_high_volt(Tango::DevDouble);
		bool flag_trg_upd;
		bool flag_itm_upd;
		bool flag_rng_upd;
		bool flag_naq_upd;
		bool flag_acq_upd;
		bool flag_hvs_upd;
		bool flag_hvs_set_upd;
		bool flag_alarm1_upd;
		bool flag_alarm2_upd;
		omni_mutex::omni_mutex *mutex;
		unsigned char comm_error_mask;

		double state_tout;

};

}	// namespace_ns

#endif	// _AH401B_H
